// ObserverSwarm.java
import swarm.Globals; // no # but a ; at the end
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.defobj.ZoneImpl;
import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.objectbase.Swarm;
import swarm.objectbase.VarProbe;
import swarm.objectbase.MessageProbe;
import swarm.objectbase.EmptyProbeMapImpl;
import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.ZoomRasterImpl;
import swarm.analysis.EZGraph;
import swarm.analysis.EZGraphImpl;
import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;
import swarm.defobj.LispArchiverImpl;

public class ObserverSwarm extends GUISwarmImpl
	{
// declare variables
	public int displayFrequency;
	public int displayConsumerName;

	public ActionGroup displayActions;
	public Schedule displaySchedule;

	public ModelSwarm modelSwarm;

	public EZGraphImpl spendingGraph;
	public EZGraphImpl consumerGraph;

// constructor
	public ObserverSwarm(Zone aZone)
		{
		super(aZone);
// create probe map
		EmptyProbeMapImpl probeMap=new EmptyProbeMapImpl(getZone(),getClass());
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("displayFrequency",getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("displayConsumerName",getClass()));
		Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());
		}// end of constructor
		
// build Objects
	public Object buildObjects()
		{
		LispArchiverImpl archiver;
		
		super.buildObjects();
// create the archiver object
		archiver=new LispArchiverImpl(getZone(),"parameters.scm");
// import the parameters (note the casting as ModelSwarm). The archiver
// will also use automatically the constructor for the ModelSwarm
		modelSwarm=(ModelSwarm)archiver.getWithZone$key(getZone(),"modelSwarm");
// create probe displays
	    Globals.env.createArchivedProbeDisplay(modelSwarm,"modelSwarm");
    	Globals.env.createArchivedProbeDisplay(this, "observerSwarm");
// set control panel to stop
		getControlPanel().setStateStopped();
// build models objects
		modelSwarm.buildObjects();
// create graphs
// spendingGraph shows the average, total and minimum spending
// of consumers during the game
// note the ease of use of the constructor:
//public EZGraphImpl(Zone aZone,
//                   String aTitle,
//                   String xl,
//                   String yl,
//                   String windowGeometryRecordName)
		spendingGraph=new EZGraphImpl(getZone(),"Agents' spending",
				"Timex2","Spending","spendingGraph");
// create the average,total,min sequences (remember the try/catch blocks
// necessary with selectors)
	    try
			{
      		spendingGraph.createAverageSequence$withFeedFrom$andSelector 
        		("Average spending",modelSwarm.getListOfConsumers(),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
      		spendingGraph.createTotalSequence$withFeedFrom$andSelector 
        		("Total spending",modelSwarm.getListOfConsumers(),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
      		spendingGraph.createMinSequence$withFeedFrom$andSelector 
        		("Minimum spending",modelSwarm.getListOfConsumers(),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
    		}
		catch (Exception e)
			{
      		System.err.println ("Exception in creating spendingGraph: " 
                          + e.getMessage ());
    		} 
// consumerGraph shows the frequentation and spending on the market
// of a consumer (determined by displayConsumerName).
		consumerGraph=new EZGraphImpl(getZone(),"A consumer",
				"Timex2","Visit/Spending","consumerGraph");
// create the sequences (remember the try/catch blocks
// necessary with selectors)
	    try
			{
      		consumerGraph.createSequence$withFeedFrom$andSelector 
        		("Went to the market",modelSwarm.getConsumer(displayConsumerName),
         		new Selector(Class.forName("Consumer"), "getVisit", 
                       false));
      		consumerGraph.createSequence$withFeedFrom$andSelector 
        		("Spent",modelSwarm.getConsumer(displayConsumerName),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
    		}
		catch (Exception e)
			{
      		System.err.println ("Exception in creating consumerGraph: " 
                          + e.getMessage ());
    		} 
		return this;
		}// end of observer buildObjects

	public Object buildActions()
		{
		super.buildActions();
		modelSwarm.buildActions();
// define displayActions
		displayActions=new ActionGroupImpl(getZone());
		try
			{
			displayActions.createActionTo$message(spendingGraph,
				new Selector(spendingGraph.getClass(),"step",true));
			displayActions.createActionTo$message(consumerGraph,
				new Selector(consumerGraph.getClass(),"step",true));
			displayActions.createActionTo$message(this,
				new Selector(getClass(),"observerCheckToStop",false));
			displayActions.createActionTo$message(getActionCache(),
				new Selector(getActionCache().getClass(),"doTkEvents",true));			}
		catch (Exception e)
			{
			e.printStackTrace(System.err);
			System.exit(1);
			}
// now schedule the actions in time
		displaySchedule=new ScheduleImpl(getZone(),2);
		displaySchedule.at$createAction(1,displayActions);
		return this;
		}// end of observer buildActions

// activity
	public Activity activateIn(Swarm swarmContext)
		{
		super.activateIn(swarmContext);
		modelSwarm.activateIn(this);
		displaySchedule.activateIn(this);
		return this.getActivity();
		}

// observerCheckToStop
	public Object observerCheckToStop()
		{
		if(modelSwarm.checkToStop()==1)
			{
			getControlPanel().setStateStopped();
			}
		return this;
		}

	}